from collections import OrderedDict


keys = OrderedDict([
    ("j", "down"),
    ("k", "up"),
    ("ctrl f", "page down"),
    ("ctrl d", "page down"),
    ("ctrl b", "page up"),
    ("ctrl u", "page up")]
)
